@@ -30,7 +30,7 @@ load_graph_draw (LoadGraph *g)
     
     if (!g->disp->window) 
     	return;
-    
+  
     g->draw_width = g->disp->allocation.width - 2 * FRAME_WIDTH;
     g->draw_height = g->disp->allocation.height - 2 * FRAME_WIDTH - 2;
     
@@ -64,7 +64,7 @@ load_graph_draw (LoadGraph *g)
 			TRUE, 0, 0,
 			g->disp->allocation.width,
 			g->disp->allocation.height);
-			
+		
     /* draw frame */
     gdk_gc_set_foreground (g->gc, &(g->colors [1]));
     gdk_draw_rectangle (g->pixmap,
@@ -104,9 +104,9 @@ load_graph_draw (LoadGraph *g)
 	}
 	g->pos[g->num_points - 1] -= g->data [g->num_points - 1] [j];
     }
-    
+   
     gdk_gc_set_line_attributes (g->gc, 1, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_MITER );
-	
+
     gdk_draw_pixmap (g->disp->window,
 		     g->disp->style->fg_gc [GTK_WIDGET_STATE(g->disp)],
 		     g->pixmap,
@@ -125,46 +125,64 @@ get_load (gfloat data [2], LoadGraph *g)
     float usr, nice, sys, free;
     float total;
     gchar *text;
-
+    gint i;
+  
     glibtop_cpu cpu;
 	
     glibtop_get_cpu (&cpu);
-	
-    
-    g->cpu_time [0] = cpu.user;
-    g->cpu_time [1] = cpu.nice;
-    g->cpu_time [2] = cpu.sys;
-    g->cpu_time [3] = cpu.idle;
-
+  
+    for (i=0; i<g->n; i++) {
+    	if (g->n == 1) {
+    		g->cpu_time [i][0] = cpu.user;
+    		g->cpu_time [i][1] = cpu.nice;
+    		g->cpu_time [i][2] = cpu.sys;
+    		g->cpu_time [i][3] = cpu.idle;
+    	} else {
+    		g->cpu_time [i][0] = cpu.xcpu_user[i];
+    		g->cpu_time [i][1] = cpu.xcpu_nice[i];
+    		g->cpu_time [i][2] = cpu.xcpu_sys[i];
+    		g->cpu_time [i][3] = cpu.xcpu_idle[i];
+    	}
+    }
     if (!g->cpu_initialized) {
-	memcpy (g->cpu_last, g->cpu_time, sizeof (g->cpu_last));
+        for (i=0; i<g->n; i++) {
+    		g->cpu_last [i][0] = g->cpu_time [i][0];
+    		g->cpu_last [i][1] = g->cpu_time [i][1];
+    		g->cpu_last [i][2] = g->cpu_time [i][2];
+    		g->cpu_last [i][3] = g->cpu_time [i][3];
+    		data[i] = -1;
+    	}
 	g->cpu_initialized = 1;
+	return;
     }
 
-    usr  = g->cpu_time [0] - g->cpu_last [0];
-    nice = g->cpu_time [1] - g->cpu_last [1];
-    sys  = g->cpu_time [2] - g->cpu_last [2];
-    free = g->cpu_time [3] - g->cpu_last [3];
+    for (i=0; i<g->n; i++) {
+    	usr  = g->cpu_time [i][0] - g->cpu_last [i][0];
+    	nice = g->cpu_time [i][1] - g->cpu_last [i][1];
+    	sys  = g->cpu_time [i][2] - g->cpu_last [i][2];
+    	free = g->cpu_time [i][3] - g->cpu_last [i][3];
+    	
+    	total = usr + nice + sys + free;
 
-    total = usr + nice + sys + free;
+    	g->cpu_last [i][0] = g->cpu_time [i][0];
+    	g->cpu_last [i][1] = g->cpu_time [i][1];
+    	g->cpu_last [i][2] = g->cpu_time [i][2];
+    	g->cpu_last [i][3] = g->cpu_time [i][3];
 
-    g->cpu_last [0] = g->cpu_time [0];
-    g->cpu_last [1] = g->cpu_time [1];
-    g->cpu_last [2] = g->cpu_time [2];
-    g->cpu_last [3] = g->cpu_time [3];
+    	if (!total) total = 1.0;
 
-    if (!total) total = 1.0;
+    	usr  = usr  / total;
+    	nice = nice / total;
+    	sys  = sys  / total;
+    	free = free / total;
 
-    usr  = usr  / total;
-    nice = nice / total;
-    sys  = sys  / total;
-    free = free / total;
-
-    data[0] = usr + sys + nice;
-    
-    text = g_strdup_printf ("%.2f %s", 100.0 *(usr + sys + nice), "%");
-    gtk_label_set_text (GTK_LABEL (g->label), text);
-    g_free (text);
+    	data[i] = usr + sys + nice;
+    	
+	text = g_strdup_printf ("%.1f%%", 100.0*data[i]);
+    	gtk_label_set_text (GTK_LABEL (g->cpu_labels[i]), text);
+    	g_free (text);
+    }
+   
     
 }
 
@@ -179,19 +197,18 @@ get_memory (gfloat data [1], LoadGraph *g)
     glibtop_swap swap;
 	
     glibtop_get_mem (&mem);
+     
     glibtop_get_swap (&swap);
 	
-    user = mem.used - mem.buffer - mem.shared;
-    
     swap_used = (float)swap.used / (float)swap.total;
-	
-    user    = user   / (float)mem.total;
-    shared  = mem.shared / mem.total;
-    buffer  = mem.buffer / mem.total;
-    free    = mem.free / mem.total;
+    
+    user    = (float)mem.user / (float)mem.total;
+    shared  = (float)mem.shared / (float)mem.total;
+    buffer  = (float)mem.buffer / (float)mem.total;
+    free    = (float)mem.free / (float)mem.total;
     
     text1 = get_size_string (mem.total);
-    text2 = get_size_string (mem.used);
+    text2 = get_size_string (mem.user);
     gtk_label_set_text (GTK_LABEL (g->memused_label), text2);
     gtk_label_set_text (GTK_LABEL (g->memtotal_label), text1);
     g_free (text1);
@@ -219,7 +236,7 @@ load_graph_update (LoadGraph *g)
 
     switch (g->type) {
     case CPU_GRAPH:
-    	get_load (g->data [0], g);
+    	get_load (g->data[0], g);
     	break;
     case MEM_GRAPH:
     	get_memory (g->data [0], g);
@@ -271,7 +288,7 @@ load_graph_alloc (LoadGraph *g)
 
     if (g->allocated)
 	return;
-    
+ 
     g->data = g_new0 (gfloat *, g->num_points);
     g->odata = g_new0 (gfloat *, g->num_points);
     g->pos = g_new0 (guint, g->num_points);
@@ -321,7 +338,7 @@ load_graph_configure (GtkWidget *widget, GdkEventConfigure *event,
 		     0, 0,
 		     c->disp->allocation.width,
 		     c->disp->allocation.height);
-		     
+
     load_graph_draw (c);
     
     return TRUE;
@@ -333,7 +350,7 @@ load_graph_expose (GtkWidget *widget, GdkEventExpose *event,
 		   gpointer data_ptr)
 {
     LoadGraph *g = (LoadGraph *) data_ptr;
-    	
+   	
     gdk_draw_pixmap (widget->window,
 		     widget->style->fg_gc [GTK_WIDGET_STATE(widget)],
 		     g->pixmap,
@@ -363,21 +380,25 @@ LoadGraph *
 load_graph_new (gint type, ProcData *procdata)
 {
     LoadGraph *g;
-
+    gint i = 0;
+   
     g = g_new0 (LoadGraph, 1);
-
+    
     g->type = type;
     switch (type) {
     case CPU_GRAPH:
-    	g->n = 1;
-    	break;
+    	if (procdata->config.num_cpus == 0)
+	    g->n = 1;
+	else
+	    g->n = procdata->config.num_cpus;
+	break;
     case MEM_GRAPH:
     	g->n = 2;
-    	break;
+	break;
     }
 	
     g->speed  = procdata->config.graph_update_interval;
-    g->num_points = 200;
+    g->num_points = 100;
     
     g->colors = g_new0 (GdkColor, g->n + 2);
 
@@ -385,7 +406,8 @@ load_graph_new (gint type, ProcData *procdata)
     g->colors[1] = procdata->config.frame_color;
     switch (type) {
     case CPU_GRAPH:
-    	g->colors[2] = procdata->config.cpu_color;
+    	for (i=0;i<g->n;i++)
+    		g->colors[2+i] = procdata->config.cpu_color[i];
     	break;
     case MEM_GRAPH:
     	g->colors[2] = procdata->config.mem_color;
@@ -441,10 +463,6 @@ load_graph_start (LoadGraph *g)
 void
 load_graph_stop (LoadGraph *g)
 {
-    /*if (g->timer_index != -1)
-	gtk_timeout_remove (g->timer_index);
-    	
-    g->timer_index = -1;*/
     if (!g)
     	return;
     g->draw = FALSE;
